/*******************************************************************************

  Defines Format Plug-in Library Interface for DIMIN Viewer n5 (DIV5)

  Programmer: Dima V. Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  Plug-in Notes:
    Each DLL must export (following plugin defs):
      1) divGetPluginHeader
    Each filter DLL must export:
      1) divGetFormatHeader
      3) define pluginType in TDIVPluginHeader as DIV_PLUGIN_TYPE_FORMAT

  History:
    08/14/2004 14:43 - First Creation
    09/12/2004 23:55 - v5.3
    01/29/2005 21:49 - v5.4 Change canAddMeta -> canWriteMultiPage

  Version : 5.4

*******************************************************************************/

#ifndef DIV_FORMAT_DEFS_H
#define DIV_FORMAT_DEFS_H

#include <windows.h>
#include "DIVTypeDefs.h"

#define DIV_PLUGIN_FORMAT_ENTRY_NAME "divGetFormatHeader"

//------------------------------------------------------------------------------
// I/O Modes
//------------------------------------------------------------------------------

typedef enum {
  DIV_IO_READ   = 0,
  DIV_IO_WRITE  = 1
} TDIVIOModes;


//------------------------------------------------------------------------------
// IMAGE
//------------------------------------------------------------------------------

// resolution units follow TIFF standard units and extend for scientific apps
#define DIV_RESUNIT_NONE       0 // no meaningful units - TIFF
#define DIV_RESUNIT_TNONE      1 // no meaningful units
#define DIV_RESUNIT_INCH       2 // english (pixels per unit) - TIFF
#define DIV_RESUNIT_CENTIMETER 3 // metric  (pixels per unit) - TIFF
#define DIV_RESUNIT_METER_PIX  4 // meters  per pixel - scientific
#define DIV_RESUNIT_MICRON_PIX 5 // microns per pixel - scientific

static char* DIV_RESUNIT_TEXT[6] = { "", "", "dpi", "cm", "m", "m" };

typedef struct TDivImageInfo 
{
  DIM_UINT32  ver;           // must be == sizeof(TDivImageInfo)
  
  DIM_UINT32  width;
  DIM_UINT32  height;
  
  DIM_UINT32  number_pages;  // number of images within the file
                             // host will set it if multiple pages are to be submited for writing
  DIM_UINT32  number_levels; // number of levels in resolution pyramid
    
  DIM_UINT32  transparentIndex;
  DIM_UINT32  transparencyMatting;  // if A == 0: no matte
    
  DIM_UINT32  samples;      // Samples per pixel
  DIM_UINT32  depth;        // Bits per sample, currently must be 1,4,8 or 16
  
  DIM_UINT32  resUnits;     // resolution units defined by DIV_RESUNIT_
  DIM_DOUBLE  xRes;         // (pixels per unit) OR (units per pixel), note: depend on resUnits
  DIM_DOUBLE  yRes;         // (pixels per unit) OR (units per pixel), note: depend on resUnits
  
  DIM_UINT32  tileWidth;    // The width of the tiles. Zero if not set
  DIM_UINT32  tileHeight;   // The height of the tiles. Zero if not set
                     
} TDivImageInfo;


//------------------------------------------------------------------------------
//  META DATA
//------------------------------------------------------------------------------

typedef enum {
  DIM_META_TIFF_TAG = 0,  // any tiff tag
  DIM_META_EXIF     = 1,  // whole EXIF buffer
  DIM_META_IPTC     = 2,  // whole IPTC buffer
  DIM_META_GEO_TIFF = 3,  // whole GeoTiff buffer
  DIM_META_BIORAD   = 4,  // any of BioRad PIC tag
  DIM_META_STK      = 5,  // text formated STK tags
  DIM_META_PNG      = 6   // any of PNG text notes
} TDIVMetaGroups;

typedef struct TDivTagItem
{
  DIM_UINT32 tagGroup;  // group to which tag's ID pertense, e.g. TIFF, EXIF, BioPIC... value of (DIM_MetaGroups)
  DIM_UINT32 tagId;     // tag ID: e.g. TIFF tag number
  DIM_UINT32 tagType;   // type: format in which data is stored inside data buffer: value of DIM_DataType
  DIM_UINT32 tagLength; // how many records of type is stored in data buffer
  void       *tagData;
} TDivTagItem;

typedef struct TDivTagList
{
  DIM_UINT32   count;
  TDivTagItem  *tags;
} TDivTagList;


//****************************************************************************
// Callbacks
//****************************************************************************

// host shows progress of the executon
// done  - value in a range of [0..total]
// total - range maximum
// descr - any text describing the executed procedure
typedef void (__stdcall *TDivProgressProc)(long done, long total, char *descr);

// host shows error of the executon
// val   - error code, not treated at the moment
// descr - any text describing the error
typedef void (__stdcall *TDivErrorProc)(int val, char *descr);

// plugin should use this to test is processing should be interrupted...
// If it returns TRUE, the operation should be aborted 
typedef int (__stdcall *TDivTestAbortProc)( void );

//-----------------------------------------------------
// memory allocation prototypes, if these are null then
// formats should use standard way of memory allocation
// using malloc...
//-----------------------------------------------------

typedef void* (__stdcall *TDivMallocProc) (DIM_ULONG size);
typedef void* (__stdcall *TDivFreeProc)   (void *p);

//-----------------------------------------------------
// io prototypes, if these are null then formats should
// use standard C io functions: fwrite, fread...
//-----------------------------------------------------

// (fread)  read from stream  
typedef DIM_ULONG (__stdcall *TDivReadProc)  ( void *buffer, DIM_ULONG size, DIM_ULONG count, void *stream );
              
// (fwrite) write into stream 
typedef DIM_ULONG (__stdcall *TDivWriteProc) ( void *buffer, DIM_ULONG size, DIM_ULONG count, void *stream );

// (flush) flushes a stream
typedef DIM_INT   (__stdcall *TDivFlushProc) ( void *stream );

// (fseek)  seek within stream
typedef DIM_INT   (__stdcall *TDivSeekProc)  ( void *stream, DIM_LONG offset, DIM_INT origin );

// get file "stream" size (libtiff askes for it)
typedef DIM_ULONG (__stdcall *TDivSizeProc)  ( void *stream );

// (ftell) gets the current position in stream
typedef DIM_LONG  (__stdcall *TDivTellProc)  ( void *stream );

// (feof) tests for end-of-file on a stream
typedef DIM_INT   (__stdcall *TDivEofProc)   ( void *stream );

// (feof) tests for end-of-file on a stream
typedef DIM_INT   (__stdcall *TDivCloseProc) ( void *stream );


//******************************************************************************
// structure passed by host to a format function
//******************************************************************************
typedef struct TDivFormatHandle {
  
  DIM_UINT32 ver; // must be == sizeof(TDivFormatHandle)
  
  // IN
  TDivProgressProc      showProgressProc;    // function provided by host to show plugin progress
  TDivErrorProc         showErrorProc;       // function provided by host to show plugin error
  TDivTestAbortProc     testAbortProc;       // function provided by host to test if plugin should interrupt processing
  
  TDivMallocProc        mallocProc;          // function provided by host to allocate memory
  TDivFreeProc          freeProc;            // function provided by host to free memory

  // some standard parameters are defined here, any specific goes inside internalParams
  void                  *magic;
  DIM_UINT32            magic_length;
  DIM_UINT32            subFormat;           // sub format used to read or write, same as TDimFormatItem
  DIM_UINT32            pageNumber;          // page number to retreive/write, if number is invalid thenpage 0 is retreived
  DIM_UINT32            resolutionLevel;     // resolution level retreive/write, considered 0 (highest) as default
  DIM_UCHAR             quality;             // number between 0 and 100 to specify quality (read/write)
  DIM_UINT32            compression;         // 0 - specify lossless method
  DIM_UINT32            order;               // progressive, normal, interlaced...
  DIM_UINT32            roiX;                // Region Of Interest: Top Left X (pixels), ONLY used for reading!!!
  DIM_UINT32            roiY;                // Region Of Interest: Top Left Y (pixels), ONLY used for reading!!!
  DIM_UINT32            roiW;                // Region Of Interest: Width (pixels), ONLY used for reading!!!
  DIM_UINT32            roiH;                // Region Of Interest: Height (pixels), ONLY used for reading!!!
  TDivTagList           metaData;            // meta data tag list, ONLY used for writing, returned NULL while reading!!!
  
  void                  *imageServicesProcs; // The suite of image processing services callbacks  
  void                  *internalParams;     // internal plugin parameters

  HWND                  hwndParent;          // main window handle
  HINSTANCE             hDllInstance;        // handle to loaded dll instance (plug-in)

  char                  *fileName;           // file name
  void                  *stream;             // pointer to a file stream that might be open by host
  TDIVIOModes           io_mode;             // io mode for opened stream 
  // i/o callbacks
  TDivReadProc          readProc;            // (fread)  read from stream
  TDivWriteProc         writeProc;           // (fwrite) write into stream 
  TDivFlushProc         flushProc;           // (flush) flushes a stream
  TDivSeekProc          seekProc;            // (fseek)  seek within stream
  TDivSizeProc          sizeProc;            // () get file "stream" size (libtiff askes for it)
  TDivTellProc          tellProc;            // (ftell) gets the current position in stream
  TDivEofProc           eofProc;             // (feof) tests for end-of-file on a stream
  TDivCloseProc         closeProc;           // (fclose) close stream

  // IN/OUT
  // image part
  HBITMAP               hbmp;                // handle to BITMAP structure
  HPALETTE              hpal;                // handle to PALETTE structure
  TDivImageInfo        *imgInfo;             // pointer to image info

  void *param1; // reserved
  void *param2; // reserved
  
  char reserved[53];
  
} TDivFormatHandle;



//******************************************************************************
// plug-in provides to the host program
//******************************************************************************

//------------------------------------------------------------------------------
//  FORMAT
//------------------------------------------------------------------------------

// in all restrictions 0 means no restriction
typedef struct TDivFormatConstrains
{
  DIM_UINT32  maxWidth;
  DIM_UINT32  maxHeight;
  DIM_UINT32  maxPageNumber;
  DIM_UINT32  minSamplesPerPixel;
  DIM_UINT32  maxSamplesPerPixel;
  DIM_UINT32  minBitsPerSample;
  DIM_UINT32  maxBitsPerSample;
  DIM_BOOL    lutNotSupported;
} TDivFormatConstrains;

typedef struct TDivFormatItem
{
  char *formatNameShort; // short name, no spaces
  char *formatNameLong;  // Long format name
  char *extensions;      // pipe "|" separated supported extension list
  DIM_BOOL canRead;           // 0 - NO, 1 - YES
  DIM_BOOL canWrite;          // 0 - NO, 1 - YES
  DIM_BOOL canReadMeta;       // 0 - NO, 1 - YES
  DIM_BOOL canWriteMeta;      // 0 - NO, 1 - YES
  DIM_BOOL canWriteMultiPage; // 0 - NO, 1 - YES
  TDivFormatConstrains constrains;
  char reserved[17];
} TDivFormatItem;

typedef struct TDivFormatList
{
  DIM_UINT32        version;
  DIM_UINT32        count;
  TDivFormatItem    *item;
} TDivFormatList;


//------------------------------------------------------------------------------
// NULL for any of this function means unavailable function 
//   ex: only reader shuld return NULL for all reader functions
//------------------------------------------------------------------------------

// this wil return if image can be opened
typedef DIM_INT (__stdcall *TDivValidateFormatProc) (void *magic, DIM_UINT32 length);

//------------------------------------------------------------------------------
// FORMAT PARAMS and ALLOCATION
//------------------------------------------------------------------------------

// allocate space for parameters stuct and return it
typedef TDivFormatHandle (__stdcall *TDivAquireFormatProc) ();

// delete parameters struct
typedef void (__stdcall *TDivReleaseFormatProc) (TDivFormatHandle *fmtHndl);

typedef void (__stdcall *TDivConfigFormatProc) ( TDivFormatHandle *fmtHndl, HWND hwndParent );
typedef void (__stdcall *TDivAboutFormatProc)  ( TDivFormatHandle *fmtHndl, HWND hwndParent );

//------------------------------------------------------------------------------
// OPEN/CLOSE IMAGE PROCs
//------------------------------------------------------------------------------

typedef DIM_UINT (__stdcall *TDivOpenImageProc)  (TDivFormatHandle *fmtHndl, TDIVIOModes io_mode);
typedef void     (__stdcall *TDivCloseImageProc) (TDivFormatHandle *fmtHndl);

//------------------------------------------------------------------------------
// IMAGE INFO PROCs
//------------------------------------------------------------------------------

// image info procs
typedef DIM_UINT      (__stdcall *TDivGetNumPagesProc)  (TDivFormatHandle *fmtHndl);
// image info for each page proc
typedef TDivImageInfo (__stdcall *TDivGetImageInfoProc) (TDivFormatHandle *fmtHndl, DIM_UINT page_num);

//------------------------------------------------------------------------------
// READ/WRITE PROCs
//------------------------------------------------------------------------------

// these are the actual read/write procs
typedef DIM_UINT (__stdcall *TDivReadImageProc)  (TDivFormatHandle *fmtHndl, DIM_UINT page);
typedef DIM_UINT (__stdcall *TDivWriteImageProc) (TDivFormatHandle *fmtHndl);

typedef DIM_UINT (__stdcall *TDivReadImageTileProc)    (TDivFormatHandle *fmtHndl, DIM_ULONG tileId);
typedef DIM_UINT (__stdcall *TDivWriteImageTileProc)   (TDivFormatHandle *fmtHndl, DIM_ULONG tileId);

typedef DIM_UINT (__stdcall *TDivReadImageLineProc)    (TDivFormatHandle *fmtHndl, DIM_ULONG y);
typedef DIM_UINT (__stdcall *TDivWriteImageLineProc)   (TDivFormatHandle *fmtHndl, DIM_ULONG y);

// difference with preview is that if there's a thumbnail in the image file
// then it will be upscaled/downscaled to meet w and h...
typedef DIM_UINT (__stdcall *TDivReadImageThumbProc)   (TDivFormatHandle *fmtHndl, DIM_UINT w, DIM_UINT h);
typedef DIM_UINT (__stdcall *TDivWriteImageThumbProc)  (TDivFormatHandle *fmtHndl);

// difference with thumbnail is that if there's a thumbnail in the image file
// smaller then needed preview then actual image will be downscaled or to meet w and h,
// if image contains Resolution Levels they'll be used to retreive optimum data
// roi will be used as weel if defined in fmtHndl, if W and H are defined as 0 
// then ROI will be extracted
typedef DIM_UINT (__stdcall *TDivReadImagePreviewProc) (TDivFormatHandle *fmtHndl, DIM_UINT w, DIM_UINT h);

//------------------------------------------------------------------------------
// METADATA PROCs
//------------------------------------------------------------------------------

// if cannot read any meta then return value different form 0
// any possible group = -1, any possible tag = -1
// so to read all possible metadata pass -1 as group and tag
// if tag or group are provided but not present in the file then return 2
// Meta-data arrray and it's contents should be destroyed on TDivCloseImageProc
// and consequtively on TDivReleaseFormatProc, host should copy needed
// meta-data items to respective areas
typedef DIM_UINT (__stdcall *TDivReadMetaDataProc)    ( TDivFormatHandle *fmtHndl, DIM_UINT page, int group, int tag, int type);

// a simplistic approach to metadata retrieval, return null terminated string
// with possible new line caracters OR NULL if no data could be found
typedef char*    (__stdcall *TDivReadMetaDataAsTextProc) ( TDivFormatHandle *fmtHndl );

// if cannot write requested meta then return value different form 0
// this will only work on formats allowing adding meta data into the file
// usually it should writen using TDivWriteImageProc and meta data struct inside fmtHndl
typedef DIM_UINT (__stdcall *TDivAddMetaDataProc)     ( TDivFormatHandle *fmtHndl);


//------------------------------------------------------------------------------
// EXTENDED FUNCTIONALITY PROCs - if needed, basically future extensions
//------------------------------------------------------------------------------

typedef DIM_UINT (__stdcall *TDivXtdProc) ( TDivFormatHandle *fmtHndl, DIM_UINT id);


//------------------------------------------------------------------------------
// *** HEADER
// structure that plug-in should provide for the host program
//------------------------------------------------------------------------------

typedef struct TDivFormatHeader {
  DIM_UINT  ver;          // must be == sizeof(TDivFormatHeader)
  char *version;          // plugin version, e.g. "1.0.0"
  char *name;             // plugin name
  char *description;      // plugin description
  
  DIM_UINT                    neededMagicSize; // 0 or more, specify number of bytes needed to identify the file
  TDivFormatList              supportedFormats;
  
  TDivValidateFormatProc      validateFormatProc; // return TRUE if can handle file
  // begin
  TDivAquireFormatProc        aquireFormatProc;
  // end
  TDivReleaseFormatProc       releaseFormatProc;

  // params
  TDivAboutFormatProc         aboutFormatProc;
  TDivConfigFormatProc        configFormatProc;
  
  // image begin
  TDivOpenImageProc           openImageProc; 
  TDivCloseImageProc          closeImageProc; 

  // info
  TDivGetNumPagesProc         getNumPagesProc;
  TDivGetImageInfoProc        getImageInfoProc;


  // read/write
  TDivReadImageProc           readImageProc; 
  TDivWriteImageProc          writeImageProc;
  TDivReadImageTileProc       readImageTileProc;
  TDivWriteImageTileProc      writeImageTileProc;
  TDivReadImageLineProc       readImageLineProc;
  TDivWriteImageLineProc      writeImageLineProc;
  TDivReadImageThumbProc      readImageThumbProc;
  TDivWriteImageThumbProc     writeImageThumbProc;
  TDivReadImagePreviewProc    readImagePreviewProc;
  
  // meta data
  TDivReadMetaDataProc        readMetaDataProc;
  TDivAddMetaDataProc         addMetaDataProc;
  TDivReadMetaDataAsTextProc  readMetaDataAsTextProc;

  // args
  TDivConfigFormatProc        configInitFormatProc;
  TDivConfigFormatProc        configQuitFormatProc;

  // xtd
  TDivXtdProc                 xtdProc;
  
  void *param1;       // reserved
  void *param2;       // reserved
  void *param3;       // reserved
  char reserved[32];
} TDivFormatHeader;


// -----------------------------------------------------------------------------
// function prototypes
// -----------------------------------------------------------------------------

// This functions must be exported so plug-in could be accepted as valid by host

// divGetFormatHeader
typedef TDivFormatHeader* (__stdcall *TDivGetFormatHeader)(void);

#endif //DIV_FORMAT_DEFS_H


